#pragma once
#include "LCWin.h"
#include <d3d11.h>
#include <wrl.h>
#include "LCException.h"
#include "Colors.h"
#include "Surface.h"
#include "RectI.h"

class Graphics
{
public:
	class Exception : public LCException
	{
	public:
		Exception( HRESULT hr,const std::wstring& note,const wchar_t* file,unsigned int line );
		std::wstring GetErrorName() const;
		std::wstring GetErrorDescription() const;
		virtual std::wstring GetFullMessage() const override;
		virtual std::wstring GetExceptionType() const override;
	private:
		HRESULT hr;
	};
private:
	// vertex format for the framebuffer fullscreen textured quad
	struct FSQVertex
	{
		float x,y,z;		// position
		float u,v;			// texcoords
	};
public:
	Graphics( class HWNDKey& key );
	Graphics( const Graphics& ) = delete;
	Graphics& operator=( const Graphics& ) = delete;
    static RectI GetScreenRect();
	void EndFrame();
	void BeginFrame()
	{
		memset( pSysBuffer, 0x0, sizeof( Color ) * ScreenWidth * ScreenHeight );
	}
    Color GetPixel(int x, int y) const;
	void PutPixel( int x, int y, Color c )
	{
		pSysBuffer[ x + ( y * ScreenWidth ) ] = c;
	}
	template<typename Effect>
	void DrawSprite( int x, int y, RectI range, const Surface & surf, Effect effect )
	{
		for( int iy = range.top; iy < range.bottom; ++iy )
		{
			for( int ix = range.left; ix < range.right; ++ix )
			{
				const auto surfColor = surf.GetPixel( ix, iy );
				effect( surfColor, ix + x, iy + y, *this );
			}
		}
	}
    ~Graphics();
private:
	Microsoft::WRL::ComPtr<IDXGISwapChain>				pSwapChain;
	Microsoft::WRL::ComPtr<ID3D11Device>				pDevice;
	Microsoft::WRL::ComPtr<ID3D11DeviceContext>			pImmediateContext;
	Microsoft::WRL::ComPtr<ID3D11RenderTargetView>		pRenderTargetView;
	Microsoft::WRL::ComPtr<ID3D11Texture2D>				pSysBufferTexture;
	Microsoft::WRL::ComPtr<ID3D11ShaderResourceView>	pSysBufferTextureView;
	Microsoft::WRL::ComPtr<ID3D11PixelShader>			pPixelShader;
	Microsoft::WRL::ComPtr<ID3D11VertexShader>			pVertexShader;
	Microsoft::WRL::ComPtr<ID3D11Buffer>				pVertexBuffer;
	Microsoft::WRL::ComPtr<ID3D11InputLayout>			pInputLayout;
	Microsoft::WRL::ComPtr<ID3D11SamplerState>			pSamplerState;
	D3D11_MAPPED_SUBRESOURCE							mappedSysBufferTexture;
	Color*                                              pSysBuffer = nullptr;
public:
	static constexpr int ScreenWidth = 800;
	static constexpr int ScreenHeight = 600;
};