#include "MainWindow.h"
#include "Game.h"
#include "LCException.h"
#include <iostream>

int WINAPI wWinMain(HINSTANCE hInst, HINSTANCE, LPWSTR pArgs, INT)
{
    // Game loop paramters.
    int TICKS_PER_SECOND = 30; // Update() rate per second
    int SKIP_TICKS = 1000 / TICKS_PER_SECOND;
    int MAX_FRAMESKIP = 10;

    try
    {
        MainWindow wnd(hInst, pArgs);
        try
        {
            GetTickCount();
            Game game(wnd);
            game.Initialize();

            DWORD next_game_tick = GetTickCount();
            int loops;

            while (wnd.ProcessMessage())
            {
                loops = 0;
                while (GetTickCount() > next_game_tick && loops < MAX_FRAMESKIP)
                {
                    loops++;
                }

                game.Update();
                game.Render();
            }
        }
        catch (const LCException& e)
        {
            const std::wstring eMsg = e.GetFullMessage() +
                L"\n\nException caught at Windows message loop.";
            wnd.ShowMessageBox(e.GetExceptionType(), eMsg, MB_ICONERROR);
        }
        catch (const std::exception& e)
        {
            // need to convert std::exception what() string from narrow to wide string
            const std::string whatStr(e.what());
            const std::wstring eMsg = std::wstring(whatStr.begin(), whatStr.end()) +
                L"\n\nException caught at Windows message loop.";
            wnd.ShowMessageBox(L"Unhandled STL Exception", eMsg, MB_ICONERROR);
        }
        catch (...)
        {
            wnd.ShowMessageBox(L"Unhandled Non-STL Exception",
                L"\n\nException caught at Windows message loop.", MB_ICONERROR);
        }
    }
    catch (const LCException& e)
    {
        const std::wstring eMsg = e.GetFullMessage() +
            L"\n\nException caught at main window creation.";
        MessageBox(nullptr, eMsg.c_str(), e.GetExceptionType().c_str(), MB_ICONERROR);
    }
    catch (const std::exception& e)
    {
        // need to convert std::exception what() string from narrow to wide string
        const std::string whatStr(e.what());
        const std::wstring eMsg = std::wstring(whatStr.begin(), whatStr.end()) +
            L"\n\nException caught at main window creation.";
        MessageBox(nullptr, eMsg.c_str(), L"Unhandled STL Exception", MB_ICONERROR);
    }
    catch (...)
    {
        MessageBox(nullptr, L"\n\nException caught at main window creation.",
            L"Unhandled Non-STL Exception", MB_ICONERROR);
    }

    return 0;
}