#ifndef _PLAYERCHARACTER_H_
#define _PLAYERCHARACTER_H_

#include "Animation.h"
#include <vector>

class PlayerCharacter
{
private:
    enum class State
    {
        WalkingLeft,
        WalkingRight,
        WalkingUp,
        WalkingDown,
        IdleLeft,
        IdleRight,
        IdleUp,
        IdleDown,
        Count
    };

public:
    PlayerCharacter(const Vector2Int& position)
        :
        sprite("playerCharacter.bmp"),
        position(position)
    {
        for (int i = 0; i < (int)State::IdleLeft; i++)
        {
            animations.emplace_back(Animation(0, 64 * i, 64, 64, 4, sprite, 0.25f));
        }

        for (int i = (int)State::IdleLeft; i < (int)State::Count; i++)
        {
            animations.emplace_back(Animation(64, 64 * (i - (int)State::IdleLeft), 64, 64, 1, sprite, 0.25f));
        }
    }

    void Draw(Graphics& gfx);
    void SetDirection(Vector2Int& dir);
    void Update(float dt);

public:
    Surface sprite;
    Vector2Int position;
    Vector2Int velocity = { 0, 0 };
    Vector2Int lastDir = { 0, 0 };
    std::vector<Animation> animations;
    State iCurState = State::IdleDown;
    float speed = 5.0f;
};

#endif