#ifndef _SPRITEEFFECT_H_
#define _SPRITEEFFECT_H_

#include "Colors.h"
#include "Graphics.h"

namespace SpriteEffect
{
    class Chroma
    {
    public:
        Chroma(Color c)
            :
            chroma(c)
        {}
        void operator()(Color cSrc, int xDest, int yDest, Graphics& gfx) const
        {
            if (cSrc != chroma)
            {
                gfx.PutPixel(xDest, yDest, cSrc);
            }
        }
    private:
        Color chroma;
    };
    class Substitution
    {
    public:
        Substitution(Color c, Color s)
            :
            chroma(c),
            sub(s)
        {}
        void operator()(Color cSrc, int xDest, int yDest, Graphics& gfx) const
        {
            if (cSrc != chroma)
            {
                gfx.PutPixel(xDest, yDest, sub);
            }
        }
    private:
        Color chroma = Colors::Magenta;
        Color sub;
    };
    class Copy
    {
    public:
        void operator()(Color cSrc, int xDest, int yDest, Graphics& gfx) const
        {
            gfx.PutPixel(xDest, yDest, cSrc);
        }
    };
    class Ghost
    {
    public:
        Ghost(Color c)
            :
            chroma(c)
        {}
        void operator()(Color src, int xDest, int yDest, Graphics& gfx) const
        {
            if (src != chroma)
            {
                const Color dest = gfx.GetPixel(xDest, yDest);
                const Color blend = {
                    unsigned char((src.GetR() + dest.GetR()) / 2),
                    unsigned char((src.GetG() + dest.GetG()) / 2),
                    unsigned char((src.GetB() + dest.GetB()) / 2)
                };
                gfx.PutPixel(xDest, yDest, blend);
            }
        }
    private:
        Color chroma;
    };
}
#endif