#include "Surface.h"
#include "LCWin.h"
#include <cassert>
#include <fstream>

Surface::Surface() {
}

Surface::Surface(const std::string& fileName)
{
    std::ifstream file("Sprites/" + fileName, std::ios::binary);
    assert(file);

    BITMAPFILEHEADER bmFileHeader;
    file.read(reinterpret_cast<char*>(&bmFileHeader), sizeof(bmFileHeader));
    BITMAPINFOHEADER bmInfoHeader;
    file.read(reinterpret_cast<char*>(&bmInfoHeader), sizeof(bmInfoHeader));

    assert(bmInfoHeader.biBitCount == 24
        || bmInfoHeader.biBitCount == 32
        || bmInfoHeader.biBitCount == 8);

    assert(bmInfoHeader.biCompression == BI_RGB);

    const bool  is32b = bmInfoHeader.biBitCount == 32;
    const bool  is24b = bmInfoHeader.biBitCount == 24;
    const bool  is8b = bmInfoHeader.biBitCount == 8;

    width = bmInfoHeader.biWidth;

    int yStart;
    int yEnd;
    int deltaY; //direction

    if (bmInfoHeader.biHeight < 0)
    {
        height = -bmInfoHeader.biHeight;
        yStart = 0;
        yEnd = height;
        deltaY = 1;
    }
    else
    {
        height = bmInfoHeader.biHeight;
        yStart = height - 1;
        yEnd = -1;
        deltaY = -1;
    }

    pPixels = new Color[width * height];

    file.seekg(bmFileHeader.bfOffBits);
    const int padding = (4 - (width * 3) % 4) % 4;

    for (int y = yStart; y != yEnd; y += deltaY)
    {
        for (int x = 0; x < width; x++)
        {
            const unsigned char b = file.get();
            const unsigned char g = file.get();
            const unsigned char r = file.get();

            PutPixel(x, y, { r,g,b });

            if (is32b)
                file.seekg(1, std::ios::cur);
            if (!is32b)
                file.seekg(padding, std::ios::cur);
        }

        file.seekg(padding, std::ios::cur);
    }
}

Surface::Surface(int width, int height)
    :
    width(width),
    height(height),
    pPixels(new Color[width*height])
{
}

Surface::Surface(const Surface& rhs)
    :
    Surface(rhs.width, rhs.height)
{
    const int nPixels = width * height;
    for (int i = 0; i < nPixels; i++)
    {
        pPixels[i] = rhs.pPixels[i];
    }
}

Surface::~Surface()
{
    delete[] pPixels;
    pPixels = nullptr;
}

Surface & Surface::operator=(const Surface &rhs)
{
    width = rhs.width;
    height = rhs.height;

    delete[] pPixels;
    pPixels = new Color[width * height];

    const int nPixels = width * height;
    for (int i = 0; i < nPixels; i++)
    {
        pPixels[i] = rhs.pPixels[i];
    }
    return *this;
}

void Surface::PutPixel(int x, int y, Color c)
{
    pPixels[y * width + x] = c;
}

Color Surface::GetPixel(int x, int y) const
{
    return pPixels[y * width + x];
}

int Surface::GetWidth() const
{
    return width;
}

int Surface::GetHeight() const
{
    return height;
}

RectI Surface::GetRect() const
{
    return { 0, width, 0, height };
}