#pragma once

#include <cmath>

template<typename T>
struct Vector2_
{
public:
    Vector2_() = default;
    Vector2_(T x_in, T y_in)
        :
        x(x_in),
        y(y_in)
    {}
    template<typename S>
    explicit Vector2_(const Vector2_<S>& src)
        :
        x((T)src.x),
        y((T)src.y)
    {}

    bool operator==(const Vector2_& rhs) const
    {
        const Vector2_* ptr = dynamic_cast<const Vector2_*>(&rhs);
        if (ptr != 0)
        {
            return (x == ptr->x && y == ptr->y) && (Vector2_::operator==(*this, rhs));
        }
        else
            return false;
    }

    Vector2_ operator+(const Vector2_& rhs) const
    {
        return Vector2_(x + rhs.x, y + rhs.y);
    }
    Vector2_& operator+=(const Vector2_& rhs)
    {
        return *this = *this + rhs;
    }
    Vector2_ operator*(T rhs) const
    {
        return Vector2_(x * rhs, y * rhs);
    }
    Vector2_& operator*=(T rhs)
    {
        return *this = *this * rhs;
    }
    Vector2_ operator-(const Vector2_& rhs) const
    {
        return Vector2_(x - rhs.x, y - rhs.y);
    }
    Vector2_& operator-=(const Vector2_& rhs)
    {
        return *this = *this - rhs;
    }
    T GetLength() const
    {
        return (T)std::sqrt(GetLengthSq());
    }
    T GetLengthSq() const
    {
        return x * x + y * y;
    }
    Vector2_& Normalize()
    {
        return *this = GetNormalized();
    }
    Vector2_ GetNormalized() const
    {
        const T len = GetLength();
        if (len != (T)0)
        {
            return *this * ((T)1 / len);
        }
        return *this;
    }
public:
    T x;
    T y;
};

typedef Vector2_<float> Vector2Float;
typedef Vector2_<int>Vector2Int;