#ifndef _WORLD_H_
#define _WORLD_H_

#include "Graphics.h"
#include "Keyboard.h"
#include "Mouse.h"
#include "Vector2Int.h"
#include "TileManager.h"
#include "PlayerCharacter.h"
#include "Point.h"
#include <memory>
#include "FrameTimer.h"

class World
{
public:
    World();
    World(const World&) = delete;
    World& operator=(const World&) = delete;
    void Initialize(class Graphics& graphics);
    void Render();
    void Update(Keyboard& keyboard, Mouse& mouse);
private:
    void StepFrame();
private:
    class Graphics* graphics = nullptr;
    class TileManager tileManager;
private:
    std::unique_ptr<float[][Constants::RenderHeight]> heightmap
    {
        std::make_unique<float[][Constants::RenderHeight]>(Constants::RenderHeight)
    };
    std::unique_ptr<Point[][Constants::RenderHeight]> grid
    {
        std::make_unique<Point[][Constants::RenderHeight]>(Constants::RenderHeight)
    };
private:
    FrameTimer ft;
    int worldOffsetX = 0;
    int worldOffsetY = 0;
    Vector2Int mousePosition;
};

#endif